<?php
	/**
	 * @file config.default.php
	 *
	 * Plugin's config file, holding configurations and defaults to be used at metadata collection the registration of
	 * resources.
	 */

    global $applicationname, $search_all_workflow_states, $additional_archive_states;

	# enable search for all states
	$search_all_workflow_states = true;


	/**
	 * A list of needed permissions.
	 *
	 *
		Administration
		--------------
		a      Can access administration tree
		t      Can see the team centre home
		r      Can manage research requests
		R      Can manage resource requests / orders
		o      Can manage content
		m      Can bulk-mail users
		u      Can manage users
		k      Can manage keywords (add/remove keyword relationships and add/remove/rename checkbox/dropdownlist options)
	 *
	 *
	 * Consider http://wiki.resourcespace.org/index.php/User_Group_Management for more information on permissions.
	 */
	$doi_perms_needed = ['a'];

	/**
	 * The name of the metadata-field that holds the DOI of a resource.
	 */
	$doi_field_shortname = 'doi';

	/**
	 * The prefix of all DOIs generated by this plugin.
	 * Has to be customised in config.php.
	 */
	$doi_prefix = "";

	/**
	 * DataCite's public prefix for testing.
	 */
	$doi_test_prefix = "10.5072";

	/**
	 * Desired archive state for resources to be registrable.
	 * Maybe customised according to the already used archive states.
	 * Set also the required permissions in order to write access the resources with that state (and own assigned dois):
	 * 	UPDATE `usergroup` SET `permissions` = CONCAT(`permissions`, ',e4') WHERE (`name` = 'Administrators' OR `name` = 'Super Admin') AND `permissions` NOT LIKE '%e4%';
	 * 
	 * Set also a corresponding name: $lang['status4'] = "Immutable";
	 */
	$doi_archive_state = 4;

	/**
	 * add new archive state for immutable resources
	 */
	$additional_archive_states[] = $doi_archive_state;

	/**
	 * Query parameter in request header, to avoid pushes to DataCite's database
	 */
	$doi_use_testmode = false;

	/**
	 * Allowes pushes but only under the test prefix, resources are not resolveable
	 */
	$doi_use_testprefix = true;

	/**
	 * The default value used for the publisher field if no publisher could be extracted from the RS database.
	 * Consider the DataCite Metadata-Schema at
	 * https://schema.datacite.org/meta/kernel-3/doc/DataCite-MetadataKernel_v3.1.pdf
	 * Has to be customised in config.php.
	 */
	$doi_publisher = '';

	# DataCite & TIB account credentials
	# Has to be customised in config.php.
	$doi_username = '';
	$doi_password = '';

	# prefferd searchfields to look for mandatory DataCite fields in
	$doi_pref_creator_fields =           ['10','29'];    # Credit, Named Person(s)
	$doi_pref_title_fields =             ['8', '18'];    # Title, Caption
	$doi_pref_publicationYear_fields =   ['12'];         # Date

	# see: https://schema.datacite.org/meta/kernel-3/doc/DataCite-MetadataKernel_v3.1.pdf#page=38
	$doi_pref_creator_fields_default = '(:unac)';
	$doi_pref_title_fields_default = '(:unas)';
	$doi_pref_publicationYear_fields_default = '(:unav)';

	# log code to use when logging plugin activities in resource log & activity log
	$doi_log_code = LOG_CODE_STATUS_CHANGED;
